""" MongoData Serializer
"""
from rest_framework import serializers
from rest_framework.serializers import Serializer

from core_main_app.rest.data.serializers import XMLContentField


class MongoDataSerializer(Serializer):
    """Data serializer"""

    id = serializers.IntegerField(read_only=True)
    template = serializers.SerializerMethodField(read_only=True)
    workspace = serializers.SerializerMethodField(read_only=True)
    user_id = serializers.SerializerMethodField(read_only=True)
    title = serializers.CharField(read_only=True)
    xml_content = XMLContentField(read_only=True)
    creation_date = serializers.DateTimeField(read_only=True)
    last_modification_date = serializers.DateTimeField(read_only=True)
    last_change_date = serializers.DateTimeField(read_only=True)

    class Meta:
        """Meta"""

        fields = [
            "id",
            "template",
            "workspace",
            "user_id",
            "title",
            "xml_content",
            "creation_date",
            "last_modification_date",
            "last_change_date",
        ]

    def get_template(self, obj):
        """Return template id

        Args:
            obj:

        Returns:

        """
        return obj._template_id

    def get_workspace(self, obj):
        """Return workspace id

        Args:
            obj:

        Returns:

        """
        return obj._workspace_id

    def get_user_id(self, obj):
        """Return user id (str)

        Args:
            obj:

        Returns:

        """
        return str(obj.user_id)

    def update(self, instance, validated_data):
        pass

    def create(self, validated_data):
        pass



""" lxml URI resolver using requests api
"""
import logging

from xml_utils.xml_validation.resolvers.default_uri_resolver import (
    DefaultURIResolver,
)
from core_main_app.utils.requests_utils.requests_utils import send_get_request

logger = logging.getLogger(__name__)


class RequestsResolver(DefaultURIResolver):
    """Requests URI Resolver for lxml"""

    session_id = None

    def __init__(self, session_id=None):
        super().__init__()
        self.session_id = session_id

    def resolve(self, url, id, context):
        """Resolve the URI using the requests api

        Args:
            url:
            id:
            context:

        Returns:

        """
        try:
            response = send_get_request(
                url, cookies={"sessionid": self.session_id}
            )
            return self.resolve_string(response.content, context)
        except Exception:
            # if an error occurs return None to use the next registered resolver (or lxml default resolver)
            logger.error(
                "An error occurred with the RequestsResolver. Please make sure HTTPS is properly configured."
            )
            return None



""" Timezone Middleware
"""

from django.utils import timezone

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.user_preferences import (
    api as user_preferences_api,
)

USER_TIMEZONE_NOT_SET = "NOT_SET"


class TimezoneMiddleware:
    """TimezoneMiddleware"""

    def __init__(self, get_response):
        """Init middleware

        Args:
            get_response:
        """
        self.get_response = get_response

    def __call__(self, request):
        """Call Middleware

        Args:
            request:

        Returns:

        """
        tz_name = request.session.get("django_timezone")
        if not tz_name and request.user.id:
            try:
                user_preferences = user_preferences_api.get_by_user(
                    request.user
                )
            except DoesNotExist:
                user_preferences = None
                # Set session to avoid fetching preferences again
                request.session["django_timezone"] = USER_TIMEZONE_NOT_SET
            if user_preferences:
                tz_name = user_preferences.timezone
                request.session["django_timezone"] = tz_name

        if tz_name and tz_name != USER_TIMEZONE_NOT_SET:
            timezone.activate(tz_name)
        else:
            timezone.deactivate()
        return self.get_response(request)



""" Migration utilities
"""
import logging
from django.db import OperationalError

from django.db.migrations.recorder import MigrationRecorder


logger = logging.getLogger(__name__)


def ensure_migration_applied(app_name, migration_name):
    """Check that a given migration has been applied. Raises RuntimeError if
    it is not the case.

    Inspired by https://stackoverflow.com/a/50100972/1723284.
    """
    try:
        migrations = MigrationRecorder.Migration.objects.filter(
            app=app_name, name=migration_name
        )

        if migrations.count() != 1:
            error_message = f"Migration ({app_name}, {migration_name}) needs to be applied!"

            logger.error(error_message)
            raise RuntimeError(error_message)
    except OperationalError as exc:
        logger.warning(
            "An error occured while checking the status of the "
            f"({app_name}, {migration_name}) migration: {exc}"
        )



""" Tests Settings
"""

import os

from dotenv import load_dotenv
from django.conf import settings

# load environment variables from .env
load_dotenv()

SECRET_KEY = "fake-key"

INSTALLED_APPS = [
    # Django apps
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.sites",
    "django.contrib.staticfiles",
    # Extra apps
    "defender",
    "menu",
    # Local apps
    "core_main_app",
    "core_parser_app",
    "tests",
]

# SERVER URI
SERVER_URI = "http://127.0.0.1:8000"

# TEST DATABASE
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql_psycopg2",
        "HOST": os.environ["POSTGRES_HOST"]
        if "POSTGRES_HOST" in os.environ
        else None,
        "PORT": int(os.environ["POSTGRES_PORT"])
        if "POSTGRES_PORT" in os.environ
        else 5432,
        "NAME": os.environ["POSTGRES_DB"]
        if "POSTGRES_DB" in os.environ
        else None,
        "USER": os.environ["POSTGRES_USER"]
        if "POSTGRES_USER" in os.environ
        else None,
        "PASSWORD": os.environ["POSTGRES_PASS"]
        if "POSTGRES_PASS" in os.environ
        else None,
    }
}

MIDDLEWARE = (
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
)

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": ["templates"],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "core_main_app.utils.custom_context_processors.domain_context_processor",
                "django.template.context_processors.i18n",
            ],
        },
    },
]

LOGIN_URL = "/login"
STATIC_URL = "/static/"
ROOT_URLCONF = "tests.urls"
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"
CELERYBEAT_SCHEDULER = "django_celery_beat.schedulers:DatabaseScheduler"
MEDIA_ROOT = "tests_media"

PASSWORD_HASHERS = ("django.contrib.auth.hashers.UnsaltedMD5PasswordHasher",)

CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT = getattr(
    settings, "CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT", False
)


DATA_SORTING_FIELDS = ["+title"]

CUSTOM_NAME = "Curator"
ENABLE_SAML2_SSO_AUTH = False
VERIFY_DATA_ACCESS = False

USE_TZ = True
CHECKSUM_ALGORITHM = "MD5"
MONGODB_INDEXING = False
MONGODB_ASYNC_SAVE = False
DJANGO_SIMPLE_HISTORY_MODELS = []
BOOTSTRAP_VERSION = "5.1.3"



""" Fixtures files for TemplateXslRendering
"""

from core_main_app.components.template.models import Template
from core_main_app.components.template_xsl_rendering.models import (
    TemplateXslRendering,
)
from core_main_app.components.xsl_transformation.models import (
    XslTransformation,
)
from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)


class TemplateXslRenderingFixtures(FixtureInterface):
    """TemplateXslRendering fixtures"""

    template_1 = None
    template_2 = None
    template_3 = None
    template_xsl_rendering_1 = None
    template_xsl_rendering_2 = None
    xsl_transformation_1 = None
    xsl_transformation_2 = None
    xsl_transformation_3 = None
    template_xsl_rendering_collection = None
    xsl_transformation_collection = None

    def insert_data(self):
        """Insert a set of TemplateXslRendering and XslTransformation.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template_xsl_rendering_collection()
        self.generate_xsl_transformation_collection()

    def generate_template_xsl_rendering_collection(self):
        """Generate a TemplateXslRendering collection.

        Returns:

        """
        self.template_1 = Template(
            filename="template_1.xsd", content="content1", hash="hash1"
        )
        self.template_1.save()
        self.template_2 = Template(
            filename="template_2.xsd", content="content2", hash="hash2"
        )
        self.template_2.save()
        self.template_3 = Template(
            filename="template_3.xsd", content="content3", hash="hash3"
        )
        self.template_3.save()
        self.template_xsl_rendering_1 = TemplateXslRendering(
            template=self.template_1
        )
        self.template_xsl_rendering_1.save()
        self.xsl_transformation_3 = XslTransformation(
            name="xsl_transformation_3",
            filename="xsl_transformation_3",
            content="content_3",
        )
        self.xsl_transformation_3.save()

        self.template_xsl_rendering_2 = TemplateXslRendering(
            template=self.template_2,
            default_detail_xslt=self.xsl_transformation_3,
        )
        self.template_xsl_rendering_2.save()
        self.template_xsl_rendering_2.list_detail_xslt.set(
            [self.xsl_transformation_3]
        )

        self.template_xsl_rendering_collection = [
            self.template_xsl_rendering_1,
            self.template_xsl_rendering_2,
        ]

    def generate_xsl_transformation_collection(self):
        """Generate a XslTransformation collection.

        Returns:

        """
        self.xsl_transformation_1 = XslTransformation(
            name="xsl_transformation_1",
            filename="xsl_transformation_1",
            content="content1",
        )
        self.xsl_transformation_1.save()
        self.xsl_transformation_2 = XslTransformation(
            name="xsl_transformation_2",
            filename="xsl_transformation_2",
            content="content_2",
        )
        self.xsl_transformation_2.save()

        self.xsl_transformation_collection = [
            self.xsl_transformation_1,
            self.xsl_transformation_2,
        ]



"""Unit tests for template version manager rest api
"""
from unittest.mock import patch

from django.test import SimpleTestCase
from rest_framework import status

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.rest.template_version_manager import views
from core_main_app.rest.template_version_manager.abstract_views import (
    AbstractOrderingTemplateVersionManager,
)
from core_main_app.rest.template_version_manager.serializers import (
    TemplateVersionManagerSerializer,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestGlobalTemplateVersionManagerList(SimpleTestCase):
    """TestGlobalTemplateVersionManagerList"""

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()

    @patch.object(TemplateVersionManager, "get_global_version_managers")
    def test_get_all_returns_http_200(self, mock_get_all):
        """test_get_all_returns_http_200

        Args:
            mock_get_all:

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")
        mock_get_all.return_value = []

        # Mock
        response = RequestMock.do_request_get(
            views.GlobalTemplateVersionManagerList.as_view(), mock_user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestUserTemplateVersionManagerList(SimpleTestCase):
    """TestUserTemplateVersionManagerList"""

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()

    @patch.object(TemplateVersionManager, "get_all_version_manager_by_user_id")
    def test_get_all_returns_http_200(self, mock_get_all):
        """test_get_all_returns_http_200

        Args:
            mock_get_all:

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")
        mock_get_all.return_value = []

        # Mock
        response = RequestMock.do_request_get(
            views.UserTemplateVersionManagerList.as_view(), mock_user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateVersionManagerDetail(SimpleTestCase):
    """TestTemplateVersionManagerDetail"""

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()

    @patch.object(TemplateVersionManager, "get_by_id")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_get_returns_http_200_when_data_exists(
        self, mock_tvm_serializer_data, mock_get_by_id
    ):
        """test_get_returns_http_200_when_data_exists

        Args:
            mock_tvm_serializer_data:
            mock_get_by_id:

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1", is_superuser=True)
        mock_get_by_id.return_value = TemplateVersionManager()
        # Mock
        response = RequestMock.do_request_get(
            views.TemplateVersionManagerDetail.as_view(),
            mock_user,
            param={"pk": "id"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(TemplateVersionManager, "get_by_id")
    def test_get_returns_http_404_when_data_not_found(self, mock_get_by_id):
        """test_get_returns_http_404_when_data_not_found

        Args:
            mock_get_by_id:

        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")
        mock_get_by_id.side_effect = DoesNotExist("error")

        # Mock
        response = RequestMock.do_request_get(
            views.TemplateVersionManagerDetail.as_view(),
            mock_user,
            param={"pk": "invalid"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestAbstractOrderingTemplateVersionManager(SimpleTestCase):
    """Test Abstract Ordering Template Version Manager"""

    def setUp(self):
        """setUp

        Returns:

        """
        super().setUp()

    @patch.multiple(
        AbstractOrderingTemplateVersionManager, __abstractmethods__=set()
    )
    def test_get_abstract_ordering_returns_error_500_if_method_is_not_implemented(
        self,
    ):
        """test_get_abstract_ordering_returns_error_500_if_method_is_not_implemented

        Args:


        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")

        # Mock
        response = RequestMock.do_request_get(
            views.AbstractOrderingTemplateVersionManager.as_view(), mock_user
        )

        # Assert
        self.assertEqual(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )

    @patch.multiple(
        AbstractOrderingTemplateVersionManager, __abstractmethods__=set()
    )
    def test_patch_abstract_ordering_returns_error_500_if_method_is_not_implemented(
        self,
    ):
        """test_patch_abstract_ordering_returns_error_500_if_method_is_not_implemented

        Args:


        Returns:

        """
        # Arrange
        mock_user = create_mock_user("1")

        # Mock
        response = RequestMock.do_request_patch(
            views.AbstractOrderingTemplateVersionManager.as_view(),
            mock_user,
            data={"template_list": []},
        )

        # Assert
        self.assertEqual(
            response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR
        )



""" XslTransformation model
"""
from django.core.exceptions import ObjectDoesNotExist
from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.validators import RegexValidator
from django.db import models, IntegrityError

from core_main_app.commons import exceptions
from core_main_app.commons.regex import NOT_EMPTY_OR_WHITESPACES
from core_main_app.settings import XSLT_UPLOAD_DIR, CHECKSUM_ALGORITHM
from core_main_app.utils.checksum import compute_checksum
from core_main_app.utils.storage.storage import core_file_storage


class XslTransformation(models.Model):
    """XslTransformation object"""

    name = models.CharField(
        unique=True,
        blank=False,
        validators=[
            RegexValidator(
                regex=NOT_EMPTY_OR_WHITESPACES,
                message="Title must not be empty or only whitespaces",
                code="invalid_title",
            ),
        ],
        max_length=200,
    )
    filename = models.CharField(
        blank=False,
        validators=[
            RegexValidator(
                regex=NOT_EMPTY_OR_WHITESPACES,
                message="Title must not be empty or only whitespaces",
                code="invalid_title",
            ),
        ],
        max_length=200,
    )
    file = models.FileField(
        blank=False,
        max_length=250,
        upload_to=XSLT_UPLOAD_DIR,
        storage=core_file_storage(model="xsl_transformation"),
    )
    checksum = models.CharField(
        max_length=512, blank=True, default=None, null=True
    )
    _content = None

    @property
    def content(self):
        """Read XSLT content

        Returns:

        """
        if not self._content:
            self._content = self.file.read().decode("utf-8")
        return self._content

    @content.setter
    def content(self, xsd_content):
        """Set xslt content

        Args:
            xsd_content:

        Returns:

        """
        # Set template content
        self._content = xsd_content

    def __str__(self):
        """String representation of an object.

        Returns:
            String representation

        """
        return self.name

    @staticmethod
    def get_all():
        """Get all XSL Transformations.

        Returns:

        """
        return XslTransformation.objects.all()

    @staticmethod
    def get_by_id_list(list_id):
        """Return the object with the given list id.

        Args:
            list_id:

        Returns:
            Object collection
        """
        return XslTransformation.objects.filter(pk__in=list_id)

    @staticmethod
    def get_by_name(xslt_name):
        """Get XSL Transformation by name.

        Args:
            xslt_name:

        Returns:

        """
        try:
            return XslTransformation.objects.get(name=xslt_name)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_by_id(xslt_id):
        """Get an XSLT document by its id.

        Args:
            xslt_id: Id.

        Returns:
            XslTransformation object.

        """
        try:
            return XslTransformation.objects.get(pk=xslt_id)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    def save_object(self):
        """Custom save.

        Returns:
            Saved Instance.

        """
        try:
            if self._content:
                self.file = SimpleUploadedFile(
                    name=self.filename,
                    content=self._content.encode("utf-8"),
                    content_type="application/xml",
                )
            if self.content and CHECKSUM_ALGORITHM:
                self.checksum = compute_checksum(
                    self.content.encode(), CHECKSUM_ALGORITHM
                )
            self.clean()
            return self.save()
        except IntegrityError as exception:
            raise exceptions.NotUniqueError(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    def clean(self):
        """Clean is called before saving

        Returns:

        """
        self.name = self.name.strip()
        self.filename = self.filename.strip()



""" Tests View Data utils
"""
from unittest import TestCase
from unittest.mock import MagicMock, patch

from django.test import override_settings
from tests.components.template_xsl_rendering.fixtures.fixtures import (
    TemplateXslRenderingFixtures,
)

from core_main_app.components.data.models import Data
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.view_builders.data import (
    _get_field,
    xslt_selector,
    build_page,
)

fixture_template_rendering = TemplateXslRenderingFixtures()


class TestGetField(TestCase):
    """Test _get_field"""

    def setUp(self) -> None:
        self.title = "test"

    def test_page_title_set_when_when_in_data(self):
        """test_page_title_set_when_when_in_data

        Returns:

        """
        # Arrange
        data = Data(title=self.title)
        # Act
        data_title = _get_field(data, "title")
        # Assert
        self.assertEqual(data_title, self.title)

    def test_page_title_empty_when_not_set_in_data(self):
        """test_page_title_None_when_not_when_in_data

        Returns:

        """
        # Arrange
        data = Data()
        # Act
        data_title = _get_field(data, "title")
        # Assert
        self.assertEqual(data_title, "")

    def test_page_title_None_when_not_in_dict(self):
        """test_page_title_None_when_not_in_dict

        Returns:

        """
        # Arrange
        data = {}
        # Act
        data_title = _get_field(data, "title")
        # Assert
        self.assertEqual(data_title, None)

    def test_page_title_set_when_in_dict(self):
        """test_page_title_set_when_in_dict

        Returns:

        """
        # Arrange
        data = {"title": self.title}
        # Act
        data_title = _get_field(data, "title")
        # Assert
        self.assertEqual(data_title, self.title)

    def test_page_title_None_when_not_data_is_None(self):
        """test_page_title_None_when_not_data_is_None

        Returns:

        """
        # Arrange
        data = None
        # Act
        data_title = _get_field(data, "title")
        # Assert
        self.assertEqual(data_title, None)

    def test_get_bad_field_return_None(self):
        """test_get_bad_field_return_None

        Returns:

        """
        # Arrange
        data = {"title": self.title}
        # Act
        data_title = _get_field(data, "bad")
        # Assert
        self.assertEqual(data_title, None)


class TestXsltSelector(IntegrationBaseTestCase):
    """Test raw_xml_to_dict"""

    fixture = fixture_template_rendering

    def test_xslt_selector_returns_default_config_when_template_xsl_rendering_is_not_set(
        self,
    ):
        """test_get_default_config_when_template_xsl_rendering_is_not_set

        Returns:

        """
        # Arrange # Act
        (
            display_xslt_selector,
            template_xsl_rendering,
            xsl_transformation_id,
        ) = xslt_selector(self.fixture.template_3.id)
        # Assert
        self.assertEqual(display_xslt_selector, False)
        self.assertEqual(template_xsl_rendering, None)
        self.assertEqual(xsl_transformation_id, None)

    def test_xslt_selector_with_only_template_xsl_rendering(self):
        """test_page_title_set_when_when_in_data

        Returns:

        """
        # Arrange # Act
        (
            display_xslt_selector,
            template_xsl_rendering,
            xsl_transformation_id,
        ) = xslt_selector(self.fixture.template_1.id)
        # Assert
        self.assertEqual(display_xslt_selector, False)
        self.assertEqual(
            template_xsl_rendering, self.fixture.template_xsl_rendering_1
        )
        self.assertEqual(xsl_transformation_id, None)

    def test_xslt_selector_returns_correct_config(self):
        """test_xslt_selector_returns_correct_config

        Returns:

        """
        # Arrange # Act
        (
            display_xslt_selector,
            template_xsl_rendering,
            xsl_transformation_id,
        ) = xslt_selector(self.fixture.template_2.id)
        # Assert
        self.assertEqual(display_xslt_selector, False)
        self.assertEqual(
            template_xsl_rendering, self.fixture.template_xsl_rendering_2
        )
        self.assertEqual(
            xsl_transformation_id, self.fixture.xsl_transformation_3.id
        )


class TestBuildPage(TestCase):
    @override_settings(INSTALLED_APPS=["core_linked_records_app"])
    @patch("core_linked_records_app.system.pid_settings.api.get")
    @patch("core_main_app.utils.view_builders.data._get_field")
    def test_build_page_with_linked_records_app_installed_calls_api(
        self, mock_get_field, mock_system_get_pid_settings
    ):
        # Arrange
        data = MagicMock()
        data.title = "title"
        mock_get_field.return_value = "test"
        mock_pid_settings = MagicMock()
        mock_pid_settings.auto_set_pid = False
        mock_system_get_pid_settings.return_value = mock_pid_settings
        # Act
        build_page(data)
        # Assert
        self.assertTrue(mock_system_get_pid_settings.called)



""" REST views for the User API
"""
from django.contrib.auth.models import User
from rest_framework import permissions
from rest_framework.generics import (
    ListCreateAPIView,
    RetrieveUpdateDestroyAPIView,
)

from core_main_app.rest.user.serializers import UserSerializer


class UserListCreateView(ListCreateAPIView):
    """User List Create View"""

    permission_classes = (permissions.IsAdminUser,)

    queryset = User.objects.all()
    serializer_class = UserSerializer


class UserRetrieveUpdateView(RetrieveUpdateDestroyAPIView):
    """User Retrieve Update View"""

    permission_classes = (permissions.IsAdminUser,)

    queryset = User.objects.all()
    serializer_class = UserSerializer



""" Apps file for setting core package when app is ready.
"""
import sys

from django.apps import AppConfig
from django.conf import settings


class InitApp(AppConfig):
    """Core application settings."""

    verbose_name = "Core Main App"

    name = "core_main_app"
    """ :py:class:`str`: Package name
    """

    def ready(self):
        from core_main_app.permissions import discover
        from core_main_app.settings import SSL_CERTIFICATES_DIR
        from core_main_app.utils.requests_utils.ssl import (
            check_ssl_certificates_dir_setting,
        )

        """When the app is ready, run the discovery and init the indexes."""
        if "migrate" not in sys.argv:
            _check_settings()
            check_ssl_certificates_dir_setting(SSL_CERTIFICATES_DIR)
            discover.init_rules(self.apps)
            discover.create_public_workspace()
            discover.init_mongo_indexing()


def _check_settings():
    """Check settings

    Returns:

    """
    from core_main_app.commons.exceptions import CoreError

    # check celery settings
    if (
        settings.CELERYBEAT_SCHEDULER
        != "django_celery_beat.schedulers:DatabaseScheduler"
    ):
        raise CoreError(
            "CELERYBEAT_SCHEDULER setting needs to be set "
            "to 'django_celery_beat.schedulers:DatabaseScheduler'."
        )



""" Unit tests for `core_main_app.templatetags.auth_extras`
"""
from unittest import TestCase
from unittest.mock import patch

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.templatetags import auth_extras
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestHasPerm(TestCase):
    """Unit tests for `has_perm` template tag."""

    def setUp(self) -> None:
        """setUp"""
        self.mock_user = create_mock_user("1")
        self.mock_kwargs = {
            "user": self.mock_user,
            "permission": "core_pkg.perm_name",
        }

    @patch.object(auth_extras, "check_has_perm")
    def test_check_has_perm_called(self, mock_check_has_perm):
        """test_check_has_perm_called"""
        auth_extras.has_perm(**self.mock_kwargs)

        mock_check_has_perm.assert_called_with(
            self.mock_user, self.mock_kwargs["permission"].split(".")[1]
        )

    @patch.object(auth_extras, "check_has_perm")
    def test_check_has_perm_success_returns_true(self, mock_check_has_perm):
        """test_check_has_perm_called"""
        mock_check_has_perm.return_value = None
        self.assertTrue(auth_extras.has_perm(**self.mock_kwargs))

    @patch.object(auth_extras, "check_has_perm")
    def test_acl_error_returns_false(self, mock_check_has_perm):
        """test_check_has_perm_called"""
        mock_check_has_perm.side_effect = AccessControlError(
            "mock_check_has_perm_acl_error"
        )
        self.assertFalse(auth_extras.has_perm(**self.mock_kwargs))



""" Utils to provide xpath validation
"""
from django.core.exceptions import ValidationError

from core_main_app.commons import exceptions
from core_main_app.utils.xml import validate_xpath


def validate_xpath_list(xpath_list):
    """Validate list of xpaths

    Args:
        xpath_list:

    Returns:

    """
    item_position = 0

    for xpath in xpath_list:
        try:
            validate_xpath(xpath)
            item_position += 1
        except exceptions.XMLError as exception:
            raise ValidationError(
                "XPath syntax error (item #%d): %s"
                % (item_position, str(exception))
            )



""" Access control utilities for core_main_app.
"""
import logging

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.group import api as group_api
from core_main_app.permissions import api as permissions_api, rights

logger = logging.getLogger(__name__)


def check_has_perm(user, permission_name):
    """Check that a user has a specific permission

    Args:
        user - User: User object.
        permission_name - str: Codename of the permission.

    Raises:
        AccessControlError - If the user does not have the permission
    """
    if user.is_superuser:  # Superusers are always authorized.
        return

    try:
        permission_object = permissions_api.get_by_codename(permission_name)

        # User is anonymous, check that the anonymous group has the permission.
        if user.is_anonymous:
            has_access = group_api.get_by_name_and_permission(
                name=rights.ANONYMOUS_GROUP,
                permission_codename=permission_name,
            )
        else:  # User is registered, check that the user is authorized.
            has_access = user.has_perm(
                f"{permission_object.content_type.app_label}."
                f"{permission_object.codename}"
            )
    except Exception as exc:  # noqa, pylint: disable=broad-except
        # If an exception occurs while checking the permission, deny access and log the
        # exception message.
        logger.warning(
            "Exception raised while executing `check_has_perm`: %s", str(exc)
        )
        has_access = False

    # Raise an exception if the user does not have the proper permission.
    if not has_access:
        raise AccessControlError(
            "The user does not have enough rights to perform this action."
        )



""" Unit tests of parse_date templatetag
"""
from unittest.case import TestCase
from unittest.mock import patch

from core_main_app.templatetags.parse_date import parse_date
from core_main_app.utils.datetime import datetime_now


class TestParseDate(TestCase):
    """Test parse_date"""

    def test_parse_date_with_datetime_returns_same_value(self):
        """test_parse_date_with_datetime_returns_same_value

        Returns:

        """
        date = datetime_now()
        self.assertEqual(date, parse_date(date))

    def test_parse_date_with_datetime_string_returns_datetime(self):
        """test_parse_date_with_datetime_string_returns_datetime

        Returns:

        """
        date = datetime_now()
        date_str = str(date)
        self.assertEqual(date, parse_date(date_str))

    def test_parse_date_with_invalid_datetime_string_returns_none(self):
        """test_parse_date_with_invalid_datetime_string_returns_none

        Returns:

        """
        date = datetime_now()
        date_str = str(date) + "bad"
        self.assertIsNone(parse_date(date_str))

    @patch("django.utils.dateparse.parse_datetime")
    def test_parse_date_with_value_error_returns_none(
        self, mock_parsedatetime
    ):
        """test_parse_date_with_value_error_returns_none

        Returns:

        """
        date = datetime_now()
        date_str = str(date)
        mock_parsedatetime.side_effect = ValueError()
        self.assertIsNone(parse_date(date_str))



"""
 Lock model
"""
import threading

from django.core.exceptions import ObjectDoesNotExist
from django.db import models

from core_main_app.commons import exceptions
from core_main_app.components.data.models import Data
from core_main_app.utils.datetime import datetime_now

sem = threading.Semaphore()


class Lock:
    """
    Class Lock. Singleton thread safe.
    Only this object should be called to be used for an action regarding locking a Document.
    """

    __singleton_lock = threading.Lock()
    __singleton_instance = None

    @classmethod
    def acquire(cls):
        """acquire.

        Args:
            cls:

        Returns:
        """
        if not cls.__singleton_instance:
            with cls.__singleton_lock:
                if not cls.__singleton_instance:
                    cls.__singleton_instance = cls()
        sem.acquire()
        return cls.__singleton_instance

    @classmethod
    def release(cls):
        """release.

        Args:
            cls:

        Returns:
        """
        sem.release()

    def set_lock(self, object, user):
        """Set lock on a Document.

        Args:
            object:
            user:

        Returns:
        """
        database_lock_object = DatabaseLockObject()
        database_lock_object.object = object
        database_lock_object.user_id = str(user.id)
        database_lock_object.lock_date = datetime_now()
        database_lock_object.save()

    def remove_lock(self, database_lock_object):
        """Remove a lock.

        Args:
            database_lock_object:

        Returns:
        """
        database_lock_object.delete()

    def get_object_locked(self, object):
        """Get the locked object.

        Args:
            object:
        Returns:
        """
        try:
            return DatabaseLockObject.get_lock_by_object(object)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))


class DatabaseLockObject(models.Model):
    """
    Class DatabaseLockObject.
    """

    object = models.ForeignKey(Data, blank=False, on_delete=models.CASCADE)
    user_id = models.CharField(blank=False, max_length=200)
    lock_date = models.DateTimeField(blank=False)

    @staticmethod
    def get_lock_by_object(obj):
        """Get lock relative to the given object.

        Args:
            obj:

        Returns:

        """
        return DatabaseLockObject.objects.get(object=obj)

    def __str__(self):
        """Database Lock as string

        Returns:

        """
        return str(self.object)



"""
    Common ajax
"""

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http.response import JsonResponse
from django.urls import reverse_lazy
from django.utils.decorators import method_decorator
from django.views.generic import CreateView, UpdateView, DeleteView

from core_main_app.commons import exceptions
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.views.admin.forms import EditTemplateForm


class AddObjectModalView(CreateView):
    """Common AddObjectModalView.
    Should be used with add.html and add.js.
    """

    template_name = "core_main_app/common/commons/form.html"
    form_class = None
    model = None
    success_url = None
    success_message = None

    def form_invalid(self, form):
        """form_invalid

        Args:
            form:

        Return:
        """
        # Get initial response
        response = super().form_invalid(form)
        data = {"is_valid": False, "responseText": response.rendered_content}
        return JsonResponse(data)

    def form_valid(self, form):
        """form_valid

        Args:
            form:

        Return:
        """
        # This method is called when valid form data has been POSTed.
        # Call private save method
        # Populate self.object without committing to the database
        self.object = form.save(commit=False)
        self._save(form)
        if not form.is_valid():
            return self.form_invalid(form)

        data = {
            "is_valid": True,
            "url": self.get_success_url(),
        }

        if self.success_message:
            messages.success(self.request, self.success_message)

        return JsonResponse(data)

    def _save(self, form):
        # Save treatment.
        super().form_valid(form)

    @staticmethod
    def get_modal_html_path():
        """get_modal_html_path

        Args:

        Return:
        """
        return "core_main_app/common/modals/add_page_modal.html"

    @staticmethod
    def get_modal_js_path():
        """get_modal_js_path

        Args:

        Return:
        """
        return {
            "path": "core_main_app/common/js/modals/add.js",
            "is_raw": False,
        }


class EditObjectModalView(UpdateView):
    """Common EditObjectModalView.
    Should be used with edit_page_modal.html and edit.js.
    """

    template_name = "core_main_app/common/commons/form.html"
    form_class = None
    model = None
    success_url = None
    success_message = None

    def form_invalid(self, form):
        """form_invalid

        Args:
            form:

        Return:
        """
        # Get initial response
        response = super().form_invalid(form)
        data = {"is_valid": False, "responseText": response.rendered_content}
        return JsonResponse(data)

    def form_valid(self, form):
        """form_valid

        Args:
            form:

        Return:
        """
        # This method is called when valid form data has been POSTed.
        # Call private save method
        self._save(form)
        if form.is_valid():
            data = {
                "is_valid": True,
                "url": self.get_success_url(),
            }

            if self.success_message:
                messages.success(self.request, self.success_message)

            return JsonResponse(data)
        else:
            return self.form_invalid(form)

    def _save(self, form):
        # Save treatment.
        super().form_valid(form)

    @staticmethod
    def get_modal_html_path():
        """get_modal_html_path

        Args:

        Return:
        """
        return "core_main_app/common/modals/edit_page_modal.html"

    @staticmethod
    def get_modal_js_path():
        """get_modal_js_path

        Args:

        Return:
        """
        return {
            "path": "core_main_app/common/js/modals/edit.js",
            "is_raw": False,
        }


class DeleteObjectModalView(DeleteView):
    """Common DeleteObjectModalView.
    Should be used with delete_page_modal.html and delete.js.
    """

    template_name = "core_main_app/common/commons/form_delete.html"
    model = None
    field_for_name = None
    success_url = None
    success_message = None

    def delete(self, request, *args, **kwargs):
        """Delete method.

        Args:
            request:

        Returns:
        """
        try:
            self.object = self.get_object()
            self._delete(request, *args, **kwargs)

            if self.success_message:
                messages.success(self.request, self.success_message)
        except Exception as exception:
            messages.error(self.request, str(exception))

        data = {"url": self.get_success_url()}
        return JsonResponse(data)

    def _delete(self, request, *args, **kwargs):
        """
        Delete treatment.
        """
        # Delete treatment.
        super().delete(request, *args, **kwargs)

    def _get_object_name(self):
        """
        Get object name

        Returns:
        """
        object_name = ""
        if self.object:
            if self.field_for_name:
                object_name = getattr(
                    self.object, self.field_for_name, str(self.object)
                )
            else:
                object_name = str(self.object)

        return object_name

    def get_context_data(self, **kwargs):
        """get_context_data

        Args:

        Return:
        """
        # Call the base implementation first to get a context
        context = super().get_context_data(**kwargs)
        # Add the object representation
        context["object_name"] = self._get_object_name()

        return context

    @staticmethod
    def get_modal_html_path():
        """get_modal_html_path

        Args:

        Return:
        """
        return "core_main_app/common/modals/delete_page_modal.html"

    @staticmethod
    def get_modal_js_path():
        """get_modal_js_path

        Args:

        Return:
        """
        return {
            "path": "core_main_app/common/js/modals/delete.js",
            "is_raw": False,
        }


@method_decorator(login_required, name="dispatch")
class EditTemplateVersionManagerView(EditObjectModalView):
    form_class = EditTemplateForm
    model = TemplateVersionManager
    success_url = reverse_lazy("core-admin:core_main_app_templates")
    success_message = "Name edited with success."

    def _save(self, form):
        # Save treatment.
        try:
            template_version_manager_api.edit_title(
                self.object,
                form.cleaned_data.get("title"),
                request=self.request,
            )
        except exceptions.NotUniqueError:
            form.add_error(
                None,
                "An object with the same name already exists. Please choose "
                "another name.",
            )
        except Exception as exception:
            form.add_error(None, str(exception))



""" Unit tests for `core_main_app.access_control.utils`.
"""
from unittest import TestCase
from unittest.mock import patch, MagicMock

from core_main_app.access_control import utils as acl_utils
from core_main_app.permissions import rights
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestCheckHasPerm(TestCase):
    """Unit tests for check_has_perm_function."""

    def setUp(self) -> None:
        """setUp"""
        self.mock_user = create_mock_user("1")

        self.mock_kwargs = {
            "user": self.mock_user,
            "permission_name": "mock_permission",
        }

    def test_superuser_does_not_call_has_perm(self):
        """test_superuser_does_not_call_has_perm"""
        super_user = create_mock_user("1", is_superuser=True)
        self.mock_kwargs["user"] = super_user

        acl_utils.check_has_perm(**self.mock_kwargs)
        super_user.has_perm.assert_not_called()

    @patch.object(acl_utils, "permissions_api")
    def test_permission_get_by_codename_is_called(self, mock_permissions_api):
        """test_permission_get_by_codename_is_called"""
        with self.assertRaises(AccessControlError):
            acl_utils.check_has_perm(**self.mock_kwargs)

        mock_permissions_api.get_by_codename.assert_called_with(
            self.mock_kwargs["permission_name"]
        )

    @patch.object(acl_utils, "logger")
    @patch.object(acl_utils, "permissions_api")
    def test_permission_get_by_codename_exception_raises_acl_error(
        self, mock_permissions_api, mock_logger
    ):
        """test_permission_get_by_codename_exception_raises_acl_error"""
        mock_permissions_api.get_by_codename.side_effect = Exception(
            "mock_get_by_codename_exception"
        )

        with self.assertRaises(AccessControlError):
            acl_utils.check_has_perm(**self.mock_kwargs)

        mock_logger.warning.assert_called()

    @patch.object(acl_utils, "group_api")
    @patch.object(acl_utils, "permissions_api")
    def test_get_by_name_and_permission_is_called(
        self, mock_permissions_api, mock_group_api
    ):
        """test_get_by_name_and_permission_is_called"""
        mock_permission_object = MagicMock()
        mock_permissions_api.get_by_codename.return_value = (
            mock_permission_object
        )
        self.mock_user.is_anonymous = True

        acl_utils.check_has_perm(**self.mock_kwargs)
        mock_group_api.get_by_name_and_permission.assert_called_with(
            name=rights.ANONYMOUS_GROUP,
            permission_codename=self.mock_kwargs["permission_name"],
        )

    @patch.object(acl_utils, "logger")
    @patch.object(acl_utils, "group_api")
    @patch.object(acl_utils, "permissions_api")
    def test_get_by_name_and_permission_exception_raises_acl_error(
        self, mock_permissions_api, mock_group_api, mock_logger
    ):
        """test_get_by_name_and_permission_exception_raises_acl_error"""
        mock_permission_object = MagicMock()
        mock_permissions_api.get_by_codename.return_value = (
            mock_permission_object
        )
        self.mock_user.is_anonymous = True
        mock_group_api.get_by_name_and_permission.side_effect = Exception(
            "mock_get_by_name_and_permission_exception"
        )

        with self.assertRaises(AccessControlError):
            acl_utils.check_has_perm(**self.mock_kwargs)

        mock_logger.warning.assert_called()

    @patch.object(acl_utils, "group_api")
    @patch.object(acl_utils, "permissions_api")
    def test_anonymous_cannot_access_raise_exception(
        self, mock_permissions_api, mock_group_api
    ):
        """test_anonymous_cannot_access_raise_exception"""
        mock_permission_object = MagicMock()
        mock_permissions_api.get_by_codename.return_value = (
            mock_permission_object
        )
        mock_group_api.get_by_name_and_permission.return_value = []
        self.mock_user.is_anonymous = True

        with self.assertRaises(AccessControlError):
            acl_utils.check_has_perm(**self.mock_kwargs)

    @patch.object(acl_utils, "group_api")
    @patch.object(acl_utils, "permissions_api")
    def test_anonymous_can_access_returns_none(
        self, mock_permissions_api, mock_group_api
    ):
        """test_anonymous_can_access_returns_none"""
        mock_permission_object = MagicMock()
        mock_permissions_api.get_by_codename.return_value = (
            mock_permission_object
        )
        mock_group_api.get_by_name_and_permission.return_value = [MagicMock()]
        self.mock_user.is_anonymous = True

        self.assertIsNone(acl_utils.check_has_perm(**self.mock_kwargs))

    @patch.object(acl_utils, "permissions_api")
    def test_user_has_perm_is_called(self, mock_permissions_api):
        """test_user_has_perm_is_called"""
        mock_permission_object = MagicMock()
        mock_permissions_api.get_by_codename.return_value = (
            mock_permission_object
        )

        with self.assertRaises(AccessControlError):
            acl_utils.check_has_perm(**self.mock_kwargs)

        self.mock_kwargs["user"].has_perm.assert_called_with(
            f"{mock_permission_object.content_type.app_label}."
            f"{mock_permission_object.codename}"
        )

    @patch.object(acl_utils, "logger")
    @patch.object(acl_utils, "permissions_api")
    def test_user_has_perm_exception_raises_acl_error(
        self, mock_permissions_api, mock_logger
    ):
        """test_user_has_perm_exception_raises_acl_error"""
        mock_permission_object = MagicMock()
        mock_permissions_api.get_by_codename.return_value = (
            mock_permission_object
        )
        self.mock_kwargs["user"].has_perm.side_effect = Exception(
            "mock_user_has_perm_exception"
        )

        with self.assertRaises(AccessControlError):
            acl_utils.check_has_perm(**self.mock_kwargs)

        mock_logger.warning.assert_called()

    @patch.object(acl_utils, "permissions_api")
    def test_registered_user_has_perm_false_raises_exception(
        self, mock_permissions_api  # noqa, pylint: disable=unused-argument
    ):
        """test_registered_user_has_perm_false_raises_exception"""
        self.mock_kwargs["user"].has_perm.return_value = False

        with self.assertRaises(AccessControlError):
            acl_utils.check_has_perm(**self.mock_kwargs)

    @patch.object(acl_utils, "permissions_api")
    def test_registered_user_has_perm_true_returns_none(
        self, mock_permissions_api  # noqa, pylint: disable=unused-argument
    ):
        """test_registered_user_has_perm_true_returns_none"""
        self.mock_kwargs["user"].has_perm.return_value = True

        self.assertIsNone(acl_utils.check_has_perm(**self.mock_kwargs))



""" MongoDB
"""
from django.conf import settings

from core_main_app.settings import (
    GRIDFS_STORAGE,
)

MONGO_CLIENT = None

# Connect to MongoDB if indexing or file storage enabled
if settings.MONGODB_INDEXING or GRIDFS_STORAGE:
    from mongoengine import connect

    from core_main_app.settings import (
        MONGO_USER,
        MONGO_PASS,
        MONGO_HOST,
        MONGO_PORT,
        MONGO_DB,
    )

    MONGODB_URI = f"mongodb://{MONGO_USER}:{MONGO_PASS}@{MONGO_HOST}:{MONGO_PORT}/{MONGO_DB}"

    MONGO_CLIENT = connect(host=MONGODB_URI, connect=False)



""" Utils for CDCS file storage
"""
from django.core.files.storage import default_storage

from core_main_app.commons.exceptions import CoreError
from core_main_app.settings import GRIDFS_STORAGE, CUSTOM_FILE_STORAGE


def user_directory_path(instance, filename):
    """Get path to user directory

    Args:
        instance:
        filename:

    Returns:

    """
    user_id = getattr(instance, "user_id", "undefined")
    # file will be uploaded to MEDIA_ROOT/user_<id>/<filename>
    return f"user_{user_id}/{filename}"


def core_file_storage(model):
    """Return file storage

    Returns:

    """
    # check if custom storage selected for model
    if model in CUSTOM_FILE_STORAGE:
        # if GridFS storage
        if (
            CUSTOM_FILE_STORAGE[model]
            == "core_main_app.utils.storage.gridfs_storage.GridFSStorage"
        ):
            # check if GridFS storage is enabled
            if GRIDFS_STORAGE:
                # return GridFS storage if enabled
                from core_main_app.utils.storage.gridfs_storage import (
                    GridFSStorage,
                )

                return GridFSStorage(collection=model)
            else:
                # raise error if not enabled
                raise CoreError(
                    f"Set GRIDFS_STORAGE to True to enable this storage for model {model}."
                )
        else:
            # return selected storage
            return CUSTOM_FILE_STORAGE[model]

    # check if GridFS storage is enabled
    if GRIDFS_STORAGE:
        # return GridFS storage if enabled
        from core_main_app.utils.storage.gridfs_storage import GridFSStorage

        return GridFSStorage(collection=model)

    # if no storage settings, return default storage
    return default_storage



""" Unit tests of get_attribute templatetag
"""
from unittest.case import TestCase

from core_main_app.templatetags.get_attribute import get_attribute


class TestObject:
    """Test object"""

    test = "test"


class TestGetAttribute(TestCase):
    """Test get_attribute"""

    def test_get_attribute_of_empty_object(self):
        """Get attribute of empty object

        Returns:

        """
        result = get_attribute({}, "test")
        self.assertEqual(result, "")

    def test_get_attribute_of_none(self):
        """Get attribute of None

        Returns:

        """
        result = get_attribute(None, "test")
        self.assertEqual(result, "")

    def test_get_attribute_of_object(self):
        """Get attribute of test object

        Returns:

        """
        test_obj = TestObject()
        result = get_attribute(test_obj, "test")
        self.assertEqual(result, "test")

    def test_get_attribute_of_dict(self):
        """Get attribute of dict

        Returns:

        """
        test_dict = {"test": "test"}
        result = get_attribute(test_dict, "test")
        self.assertEqual(result, "test")

    def test_get_attribute_of_list(self):
        """Get attribute of list

        Returns:

        """
        test_tab = ["test"]
        result = get_attribute(test_tab, "0")
        self.assertEqual(result, "test")



"""Template tag to check user permission
"""

from django import template

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.access_control.utils import check_has_perm

register = template.Library()


@register.filter(name="has_perm")
def has_perm(user, permission):
    """Check if user has the right level of permission to access a feature.

    Args:
        user:
        permission:

    Returns:
        bool: Whether the user has the permission or not.
    """
    try:
        permission_name = permission.split(".")[1]
        check_has_perm(user, permission_name)
        return True
    except AccessControlError:
        return False



""" Checksum test class
"""
from os.path import join, dirname, abspath
from unittest import TestCase

from core_main_app.commons.exceptions import CoreError
from core_main_app.utils.checksum import compute_checksum

RESOURCES_PATH = join(dirname(abspath(__file__)), "data")


class TestComputeChecksum(TestCase):
    """TestComputeChecksum"""

    def test_compute_checksum_when_algo_found(self):
        """test compute checksum when algo found

        Returns:

        """
        bytes_obj = b"this is a test."
        checksum = compute_checksum(bytes_obj, "MD5")
        self.assertIsNotNone(checksum)

    def test_compute_raises_error_if_algo_not_found(self):
        """test compute raises error if algo not found

        Returns:

        """
        bytes_obj = b"this is a test."
        with self.assertRaises(CoreError):
            compute_checksum(bytes_obj, "TEST")

    def test_compute_file_md5_returns_expected_hash(self):
        """test compute file md5 returns expected hash

        Returns:

        """
        with open(join(RESOURCES_PATH, "expand.png"), "rb") as file:
            checksum = compute_checksum(file, "MD5")
            # MD5 (expand.png) = f767652a18f716db2f0f4f5cd74b5067
            self.assertEqual(checksum, "f767652a18f716db2f0f4f5cd74b5067")



""" Integration Test for Group
"""

from tests.components.group.fixtures.fixtures import GroupFixtures
from django.contrib.auth.models import Group
from core_main_app.components.group import api as group_api
from core_main_app.utils.integration_tests.integration_base_transaction_test_case import (
    IntegrationTransactionTestCase,
)


class TestGetAllGroups(IntegrationTransactionTestCase):
    """Test Get All Groups"""

    def test_get_all_groups_returns_default_groups(self):
        """test_get_all_groups_returns_default_groups

        Returns:

        """
        # Act
        result = group_api.get_all_groups()

        # Assert
        self.assertEqual(len(result), 2)

    def test_get_all_groups_returns_group_list(self):
        """test_get_all_groups_returns_group_list

        Returns:

        """
        # Arrange
        GroupFixtures().create_group(name="test name")

        # Act
        list_groups = group_api.get_all_groups()

        # Assert
        self.assertEqual(len(list_groups), 3)
        self.assertTrue(all(isinstance(group, Group) for group in list_groups))


class TestGetGroupByNameAndPermission(IntegrationTransactionTestCase):
    """Test Get Group By Name And Permission"""

    def test_get_group_by_name_and_permission_returns_group_list(self):
        """test get group by name and permission returns group list

        Returns:

        """
        # Arrange
        group = GroupFixtures().create_group(name="test name")

        # Act
        result = group_api.get_by_name_and_permission(group.name, None)

        # Assert
        self.assertEqual(len(result), 1)
        self.assertTrue(group in result)

    def test_get_group_by_name_and_permission_returns_empty_list_if_group_does_not_exist(
        self,
    ):
        """test get group by name and permission returns empty list if group does not exist

        Returns:

        """

        # Act
        result = group_api.get_by_name_and_permission("", None)

        # Assert
        self.assertEqual(len(result), 0)

    def test_get_group_by_name_and_wrong_permission_returns_empty_list(self):
        """test get group by name and wrong permission returns empty list

        Returns:

        """

        # Arrange
        group = GroupFixtures().create_group(name="test name")

        # Act
        result = group_api.get_by_name_and_permission(group.name, "Wrong")

        # Assert
        self.assertEqual(len(result), 0)


class TestGetAllGroupByListId(IntegrationTransactionTestCase):
    """Test Get All Group By List Id"""

    def test_get_all_groups_by_list_id_returns_list(self):
        """test get all groups by list id returns list

        Returns:

        """
        # Act
        result = group_api.get_all_groups_by_list_id([1, 2])

        # Assert
        self.assertEqual(len(result), 2)

    def test_get_all_groups_by_invalid_list_id_returns_empty_list(self):
        """test get all groups by invalid list id returns empty list

        Returns:

        """
        # Act
        result = group_api.get_all_groups_by_list_id([0, -1])

        # Assert
        self.assertEqual(len(result), 0)

    def test_get_all_groups_by_list_id_returns_valid_id_group_list(self):
        """test get all groups by invalid list id returns empty list

        Returns:

        """
        # Act
        result = group_api.get_all_groups_by_list_id([1, -1])

        # Assert
        self.assertEqual(len(result), 1)


class TestGetAllGroupExceptListId(IntegrationTransactionTestCase):
    """Test Get All Group Except List Id"""

    def test_get_all_groups_except_list_id_returns_list(self):
        """test get all groups except list id returns list

        Returns:

        """
        # Arrange
        GroupFixtures().create_group(name="test name")

        # Act
        result = group_api.get_all_groups_except_list_id([1, 2])

        # Assert
        self.assertEqual(len(result), 1)

    def test_get_all_groups_except_all_available_group_ids_returns_empty_list(
        self,
    ):
        """test get all groups except all available group ids returns empty list

        Returns:

        """
        # Act
        result = group_api.get_all_groups_except_list_id([1, 2])

        # Assert
        self.assertEqual(len(result), 0)


class TestGetAllGroupExceptList(IntegrationTransactionTestCase):
    """Test Get All Group Except List"""

    def test_get_all_groups_except_list_returns_list(self):
        """test get all groups except list returns list

        Returns:

        """
        # Arrange
        group = GroupFixtures().create_group(name="test name")

        # Act
        result = group_api.get_all_groups_except_list([group])

        # Assert
        self.assertEqual(len(result), 2)



""" Integration Test for Template xsl rendering
"""

from tests.components.template_xsl_rendering.fixtures.fixtures import (
    TemplateXslRenderingFixtures,
)

from core_main_app.commons import exceptions
from core_main_app.components.template_xsl_rendering.models import (
    TemplateXslRendering,
)

from core_main_app.components.template_xsl_rendering import (
    api as template_xsl_rendering_api,
)
from core_main_app.utils.integration_tests.integration_base_transaction_test_case import (
    IntegrationTransactionTestCase,
)

fixture_template_rendering = TemplateXslRenderingFixtures()


class TestTemplateXslRenderingAddOrDelete(IntegrationTransactionTestCase):
    """Test Template Xsl Rendering Add Or Delete"""

    fixture = fixture_template_rendering

    def test_add_or_delete_returns_none_when_nothing_happened(self):
        """test_add_or_delete_returns_none_when_nothing_happened

        Args:

        Returns:

        """

        # Act
        result = template_xsl_rendering_api.add_or_delete(
            self.fixture.template_1,
            list_xslt=None,
            default_detail_xslt=None,
            list_detail_xslt=None,
        )

        # Assert
        self.assertEqual(result, None)

    def test_add_or_delete_with_given_list_xslt_creates_template_xsl_rendering(
        self,
    ):
        """test add or delete with given list xslt creates_template_xsl_rendering

        Args:

        Returns:

        """
        # Arrange
        list_xslt = self.fixture.xsl_transformation_1

        # Act
        result = template_xsl_rendering_api.add_or_delete(
            self.fixture.template_3,
            list_xslt=list_xslt,
            default_detail_xslt=None,
            list_detail_xslt=None,
        )

        # Assert
        self.assertTrue(isinstance(result, TemplateXslRendering))
        self.assertEqual(result.list_xslt, list_xslt)

    def test_add_or_delete_with_given_list_detail_xslt_creates_template_xsl_rendering(
        self,
    ):
        """test add or delete with given list detail xslt creates_template_xsl_rendering

        Args:

        Returns:

        """
        # Arrange
        list_detail_xslt = [self.fixture.xsl_transformation_1]

        # Act
        result = template_xsl_rendering_api.add_or_delete(
            self.fixture.template_3,
            list_xslt=None,
            default_detail_xslt=None,
            list_detail_xslt=list_detail_xslt,
        )

        # Assert
        self.assertTrue(isinstance(result, TemplateXslRendering))

    def test_add_or_delete_with_wrong_template_xsl_id_raises_api_error(self):
        """test add or delete with wrong template xsl id raises api error

        Args:

        Returns:

        """

        # Act # Assert
        with self.assertRaises(exceptions.ApiError):
            template_xsl_rendering_api.add_or_delete(
                self.fixture.template_3,
                list_xslt=None,
                default_detail_xslt=None,
                list_detail_xslt=None,
                template_xsl_rendering_id=-1,
            )

    def test_add_or_delete_xslt_updates_template_xsl_rendering(
        self,
    ):
        """test add or delete with given list detail xslt creates_template_xsl_rendering

        Args:

        Returns:

        """

        # Act
        result = template_xsl_rendering_api.add_or_delete(
            self.fixture.template_2,
            list_xslt=self.fixture.xsl_transformation_1,
            default_detail_xslt=None,
            list_detail_xslt=None,
            template_xsl_rendering_id=self.fixture.template_xsl_rendering_1.id,
        )

        # Assert
        self.assertTrue(isinstance(result, TemplateXslRendering))

    def test_add_or_delete_deletes_template_xsl_rendering(self):
        """test add or delete with given list detail xslt creates_template_xsl_rendering

        Args:

        Returns:

        """
        template_xsl = self.fixture.template_xsl_rendering_1
        # Act
        template_xsl_rendering_api.add_or_delete(
            self.fixture.template_3,
            list_xslt=None,
            default_detail_xslt=None,
            list_detail_xslt=None,
            template_xsl_rendering_id=template_xsl.id,
        )


class TestTemplateXslRenderingAddDetailXslt(IntegrationTransactionTestCase):
    """Test Template Xsl Rendering Add Detail Xslt"""

    fixture = fixture_template_rendering

    def test_template_xsl_add_detail_xslt_raises_error_when_already_exist_in_detail_list(
        self,
    ):
        """test template xsl add detail xslt raises error already exist in detail list

        Args:

        Returns:

        """

        # Act
        with self.assertRaises(Exception):
            template_xsl_rendering_api.add_detail_xslt(
                self.fixture.template_xsl_rendering_2,
                self.fixture.xsl_transformation_3,
            )

    def test_template_xsl_add_detail_xslt(self):
        """test template xsl add detail xslt

        Args:

        Returns:

        """

        # Act
        template_xsl_rendering_api.add_detail_xslt(
            self.fixture.template_xsl_rendering_1,
            self.fixture.xsl_transformation_1,
        )

        # Assert
        self.assertTrue(
            self.fixture.xsl_transformation_1
            in self.fixture.template_xsl_rendering_1.list_detail_xslt.all()
        )


class TestTemplateXslRenderingDeleteDetailXslt(IntegrationTransactionTestCase):
    """Test Template Xsl Rendering Delete Detail Xslt"""

    fixture = fixture_template_rendering

    def test_template_xsl_delete_detail_xslt_raises_error_when_xslt_is_not_in_detail_list(
        self,
    ):
        """test template xsl delete detail xslt raises error when xslt is not in detail list

        Args:

        Returns:

        """

        # Act
        with self.assertRaises(Exception):
            template_xsl_rendering_api.delete_detail_xslt(
                self.fixture.template_xsl_rendering_1,
                self.fixture.xsl_transformation_3,
            )

    def test_template_xsl_delete_detail_xslt_removes_xslt_from_detail_list(
        self,
    ):
        """test template xsl delete detail xslt removes xslt from detail list

        Args:

        Returns:

        """

        # Act
        template_xsl_rendering_api.delete_detail_xslt(
            self.fixture.template_xsl_rendering_2,
            self.fixture.xsl_transformation_3,
        )

        # Assert
        self.assertTrue(
            self.fixture.xsl_transformation_3
            not in self.fixture.template_xsl_rendering_2.list_detail_xslt.all()
        )


class TestTemplateXslRenderingSetDefaultDetailXslt(
    IntegrationTransactionTestCase
):
    """Test Template Xsl Rendering Set Default Detail Xslt"""

    fixture = fixture_template_rendering

    def test_template_xsl_set_default_detail_xslt_raises_error_when_xslt_is_not_in_detail_list(
        self,
    ):
        """test template xsl set default detail xslt raises error when xslt is not in detail list

        Args:

        Returns:

        """

        # Act
        with self.assertRaises(Exception):
            template_xsl_rendering_api.set_default_detail_xslt(
                self.fixture.template_xsl_rendering_1,
                self.fixture.xsl_transformation_3,
            )

    def test_template_xsl_set_default_detail_xslt_adds_xslt_as_default(self):
        """test template xsl set default detail xslt adds xslt as default

        Args:

        Returns:

        """

        # Act
        template_xsl_rendering_api.set_default_detail_xslt(
            self.fixture.template_xsl_rendering_2,
            self.fixture.xsl_transformation_3,
        )

        # Assert
        self.assertEqual(
            self.fixture.xsl_transformation_3,
            self.fixture.template_xsl_rendering_2.default_detail_xslt,
        )



""" Unit Test Validators
"""
import unittest

from django.core.exceptions import ValidationError

from core_main_app.commons.validators import (
    UpperCaseLetterCountValidator,
    LowerCaseLetterCountValidator,
    AlphabeticCharCountValidator,
    MaxOccurrenceCountValidator,
    NonAlphanumericCountValidator,
    DigitsCountValidator,
    BlankSpacesValidator,
)


class TestValidators(unittest.TestCase):
    """Test Validators"""

    def test_blank_spaces_validator_should_pass(self):
        """Test blank spaces case validator should pass

        Returns:

        """
        blank_spaces_validator = BlankSpacesValidator()
        blank_spaces_validator("PasswordTest")

    def test_blank_spaces_validator_should_fail(self):
        """Test blank spaces case validator should fail

        Returns:

        """
        blank_spaces_validator = BlankSpacesValidator()
        with self.assertRaises(ValidationError):
            blank_spaces_validator(" ")

    def test_upper_case_validator_should_pass(self):
        """Test upper case validator should pass

        Returns:

        """
        uppercase_validator = UpperCaseLetterCountValidator(2)
        uppercase_validator.validate("passWordTest")

    def test_upper_case_validator_should_fail(self):
        """Test upper case validator should fail

        Returns:

        """
        uppercase_validator = UpperCaseLetterCountValidator(2)
        with self.assertRaises(ValidationError):
            uppercase_validator.validate("passwordtest")

    def test_lower_case_validator_should_pass(self):
        """Test lower case validator should pass

        Returns:

        """
        lowercase_validator = LowerCaseLetterCountValidator(2)
        lowercase_validator.validate("PaSSWoRDTeST")

    def test_lower_case_validator_should_fail(self):
        """Test lower case validator should fail

        Returns:

        """
        lowercase_validator = LowerCaseLetterCountValidator(2)
        with self.assertRaises(ValidationError):
            lowercase_validator.validate("PASSWORDTEST")

    def test_alphabetic_validator_should_pass(self):
        """Test alphabetic validator should pass

        Returns:

        """
        alpha_validator = AlphabeticCharCountValidator(5)
        alpha_validator.validate("!@#$t%^&j&*(y(*%t$#@A*")

    def test_alphabetic_validator_should_fail(self):
        """Test alphabetic validator should fail

        Returns:

        """
        alpha_validator = AlphabeticCharCountValidator(2)
        with self.assertRaises(ValidationError):
            alpha_validator.validate("!@#$%^&&*((*%$#@*")

    def test_max_occurrence_validator_should_pass(self):
        """Test max occurrence validator should pass

        Returns:

        """
        max_occur_validator = MaxOccurrenceCountValidator(5)
        max_occur_validator.validate("abbcccdddd")

    def test_max_occurrence_validator_should_fail(self):
        """Test max occurrence validator should fail

        Returns:

        """
        max_occur_validator = MaxOccurrenceCountValidator(2)
        with self.assertRaises(ValidationError):
            max_occur_validator.validate("abbcccddddeeeee")

    def test_non_alphanumeric_validator_should_pass(self):
        """Test non alphanumeric validator should pass

        Returns:

        """
        non_alphanumeric_validator = NonAlphanumericCountValidator(5)
        non_alphanumeric_validator.validate("abbccc!@dddd$%eeee*")

    def test_non_alphanumeric_validator_should_fail(self):
        """Test non alphanumeric validator should fail

        Returns:

        """
        non_alphanumeric_validator = NonAlphanumericCountValidator(5)
        with self.assertRaises(ValidationError):
            non_alphanumeric_validator.validate("abbc*ccdd@ddee#eee")

    def test_digits_validator_should_pass(self):
        """Test digit validator should pass

        Returns:

        """
        digits_validator = DigitsCountValidator(5)
        digits_validator.validate("abbccc12345dddd")

    def test_digits_validator_should_fail(self):
        """Test digit validator should fail

        Returns:

        """
        digits_validator = DigitsCountValidator(5)
        with self.assertRaises(ValidationError):
            digits_validator.validate("abbccc1234dddd")



""" Apps test class
"""
from unittest import TestCase

from django.test import override_settings

from core_main_app.apps import _check_settings
from core_main_app.commons.exceptions import CoreError


class TestCheckSettings(TestCase):
    """TestCheckSettings"""

    @override_settings(
        CELERYBEAT_SCHEDULER="django_celery_beat.schedulers:DatabaseScheduler"
    )
    def test_check_settings_CELERYBEAT_SCHEDULER_properly_set(self):
        """test_check_settings_CELERYBEAT_SCHEDULER_properly_set

        Returns:

        """
        _check_settings()

    @override_settings(CELERYBEAT_SCHEDULER="test")
    def test_check_settings_CELERYBEAT_SCHEDULER_improperly_set_raises_error(
        self,
    ):
        """test_check_settings_CELERYBEAT_SCHEDULER_improperly_set_raises_error

        Returns:

        """
        with self.assertRaises(CoreError):
            _check_settings()



""" REST views for the blob API
"""
from abc import abstractmethod, ABCMeta

from django.http import Http404
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.permissions import (
    IsAuthenticated,
    IsAuthenticatedOrReadOnly,
    IsAdminUser,
)
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions
from core_main_app.components.blob import api as blob_api
from core_main_app.components.data import api as data_api
from core_main_app.components.user import api as user_api
from core_main_app.components.workspace import api as workspace_api
from core_main_app.rest.blob.serializers import (
    BlobSerializer,
    DeleteBlobsSerializer,
)
from core_main_app.utils.file import get_file_http_response


class AbstractBlobList(APIView, metaclass=ABCMeta):
    """Abstract Blob List"""

    @abstractmethod
    def _get_blobs(self, request):
        """Retrieve blobs

        Args:
            request:

        Returns:

        """
        raise NotImplementedError("_get_blobs method is not implemented.")

    def get(self, request):
        """Get all Blob

        Url Parameters:

            filename: document_filename

        Examples:

            ../blob/
            ../blob?filename=[filename]

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of blob
            - code: 403
              content: Authentication error
            - code: 500
              content: Internal server error
        """
        try:
            blob_list = self._get_blobs(request)

            # Apply filters
            filename = self.request.query_params.get("filename", None)
            if filename is not None:
                blob_list = blob_list.filter(filename=filename)

            # Serialize object
            serializer = BlobSerializer(
                blob_list,
                many=True,
                context={"request": request},
            )

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class BlobListAdmin(AbstractBlobList):
    """List all Blob"""

    permission_classes = (IsAdminUser,)

    def _get_blobs(self, request):
        """Retrieve blobs

        Args:
            request:

        Returns:

        """
        return blob_api.get_all(request.user)

    def get(self, request):
        """Get all accessible Blob for admin user

        Url Parameters:

            filename: document_filename

        Examples:

            ../blob/
            ../blob?filename=[filename]

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of blob
            - code: 403
              content: Authentication error
            - code: 500
              content: Internal server error
        """
        return super().get(request)


class BlobList(AbstractBlobList):
    """List all user Blob, or create a new one"""

    permission_classes = (IsAuthenticated,)

    def _get_blobs(self, request):
        """Retrieve blobs

        Args:
            request:

        Returns:

        """
        return blob_api.get_all_by_user(user=request.user)

    def get(self, request):
        """Get all blob by user

        Url Parameters:

            filename: document_filename

        Examples:

            ../blob/
            ../blob?filename=[filename]

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of blob
            - code: 403
              content: Authentication error
            - code: 500
              content: Internal server error
        """
        return super().get(request)

    def post(self, request):
        """Create Blob

        Parameters:

            {
                "blob": "[file]",
            }

        Code snippet:

            requests.post(url, files={'blob': open(BLOB_PATH, 'rb')}, auth=(USER, PSWD))

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: Created blob
            - code: 400
              content: Validation error
            - code: 500
              content: Internal server error
        """
        try:
            # Build serializer
            serializer = BlobSerializer(
                data=request.data, context={"request": request}
            )

            # Validate data
            serializer.is_valid(raise_exception=True)

            # Save data
            serializer.save(user=request.user)

            # Return the serialized data
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class BlobDetail(APIView):
    """Retrieve, update or delete a Blob"""

    permission_classes = (IsAuthenticatedOrReadOnly,)

    def get_object(self, request, pk):
        """Get Blob from db

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            Blob
        """
        try:
            return blob_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """Retrieve Blob

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content: Blob
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            blob_object = self.get_object(request, pk)

            # Serialize object
            serializer = BlobSerializer(
                blob_object, context={"request": request}
            )

            # Return response
            return Response(serializer.data)
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {"message": "Blob not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    def delete(self, request, pk):
        """Delete Blob

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 204
              content: Deletion succeed
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            blob_object = self.get_object(request, pk)

            # delete object
            blob_api.delete(blob_object, request.user)

            return Response(status=status.HTTP_204_NO_CONTENT)
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {"message": "Blob not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class BlobDownload(APIView):
    """Download Blob"""

    def get_object(self, request, pk):
        """Get Blob from db

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            Blob
        """
        try:
            return blob_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """Download the Blob file

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content: Blob file
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            blob_object = self.get_object(request, pk)

            return get_file_http_response(
                blob_object.blob.read(), blob_object.filename
            )
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {"message": "Blob not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class BlobDeleteList(APIView):
    """Delete a list of Blob"""

    permission_classes = (IsAuthenticatedOrReadOnly,)

    def patch(self, request):
        """Delete a list of Blob

        Parameters:

            [
                {
                    "id": "blob_id",
                },
                {
                    "id": "blob_id",
                }
            ]

        Args:

            request: HTTP request

        Returns:

            - code: 204
              content: Deletion succeed
            - code: 400
              content: Validation error
            - code: 403
              content: Authentication error
            - code: 500
              content: Internal server error
        """
        try:
            # Serialize data
            serializer = DeleteBlobsSerializer(
                data=request.data, many=True, context={"request": request}
            )

            # Validate data
            serializer.is_valid(raise_exception=True)

            # Get list of unique ids
            blob_ids = set([blob["id"] for blob in serializer.validated_data])

            for blob_id in blob_ids:
                # Get blob with its id
                blob = blob_api.get_by_id(blob_id, request.user)
                # Delete blob
                blob_api.delete(blob, request.user)

            # Return the serialized data
            return Response(status=status.HTTP_204_NO_CONTENT)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except AccessControlError as access_control_error:
            content = {"message": str(access_control_error)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class BlobAssign(APIView):
    """Assign a Blob to a Workspace."""

    permission_classes = (IsAuthenticated,)

    def get_object(self, request, pk):
        """Get Blob from db

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            blob
        """
        try:
            return blob_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get_workspace(self, workspace_id):
        """Retrieve a Workspace

        Args:

            workspace_id: ObjectId

        Returns:

            - code: 404
              content: Object was not found
        """
        try:
            return workspace_api.get_by_id(workspace_id)
        except exceptions.DoesNotExist:
            raise Http404

    def patch(self, request, pk, workspace_id):
        """Assign Blob to a Workspace

        Args:

            request: HTTP request
            pk: ObjectId
            workspace_id: ObjectId

        Returns:

            - code: 200
              content: None
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            blob_object = self.get_object(request, pk)
            workspace_object = self.get_workspace(workspace_id)

            # Assign blob to workspace
            blob_api.assign(blob_object, workspace_object, request.user)
            return Response({}, status=status.HTTP_200_OK)
        except Http404:
            content = {"message": "Blob or workspace not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as ace:
            content = {"message": str(ace)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class BlobChangeOwner(APIView):
    """Change the Owner of a blob"""

    permission_classes = (IsAdminUser,)

    def get_object(self, request, pk):
        """Get blob from db

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            Blob
        """
        try:
            return blob_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get_user(self, user_id):
        """Retrieve a User

        Args:

            user_id: ObjectId

        Returns:

            - code: 404
              content: Object was not found
        """
        try:
            return user_api.get_user_by_id(user_id)
        except exceptions.DoesNotExist:
            raise Http404

    def patch(self, request, pk, user_id):
        """Change the Owner of a blob

        Args:

            request: HTTP request
            pk: ObjectId
            user_id: ObjectId

        Returns:

            - code: 200
              content: None
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # get object
            data_object = self.get_object(request, pk)
            user_object = self.get_user(user_id)

            # change owner
            blob_api.change_owner(data_object, user_object, request.user)
            return Response({}, status=status.HTTP_200_OK)
        except Http404:
            content = {"message": "Blob or user not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as ace:
            content = {"message": str(ace)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


class BlobMetadata(APIView):
    """Blob Metadata"""

    permission_classes = (IsAuthenticated,)

    def get_blob(self, request, pk):
        """Get Blob from db

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            Blob
        """
        try:
            return blob_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get_metatada(self, request, metadata_id):
        """Get Data from db

        Args:

            request: HTTP request
            metadata_id: ObjectId

        Returns:

            Blob
        """
        try:
            return data_api.get_by_id(metadata_id, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def post(self, request, pk, metadata_id):
        """Add Metadata to Blob

        Args:

            request: HTTP request
            pk: Blob id
            metadata_id: Data id

        Returns:

            - code: 200
              content:
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get objects
            blob_object = self.get_blob(request, pk)
            metadata_object = self.get_metatada(request, metadata_id)
            # Update
            blob_api.add_metadata(blob_object, metadata_object, request.user)
            return Response()
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {"message": "Blob not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    def delete(self, request, pk, metadata_id):
        """Remove Metadata from Blob

        Args:

            request: HTTP request
            pk: Blob id
            metadata_id: Data id

        Returns:

            - code: 200
              content:
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get objects
            blob_object = self.get_blob(request, pk)
            metadata_object = self.get_metatada(request, metadata_id)
            # Update
            blob_api.remove_metadata(
                blob_object, metadata_object, request.user
            )
            return Response()
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {"message": "Blob not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )



""" Authentication tests for Blob REST API
"""
from io import BytesIO
from unittest.mock import patch, Mock

from django.test import SimpleTestCase
from rest_framework import status

from core_main_app.components.blob import api as blob_api
from core_main_app.components.blob.models import Blob
from core_main_app.components.workspace import api as workspace_api
from core_main_app.rest.blob import views as blob_rest_views
from core_main_app.rest.blob.serializers import (
    BlobSerializer,
    DeleteBlobsSerializer,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_main_app.components.data import api as data_api


class TestBlobListAdminGetPermissions(SimpleTestCase):
    """TestBlobListAdminGetPermissions"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_get(
            blob_rest_views.BlobListAdmin.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            blob_rest_views.BlobListAdmin.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.blob.api.get_all")
    def test_staff_returns_http_200(self, get_all):
        """test_staff_returns_http_200

        Args:
            get_all:

        Returns:

        """
        mock_user = create_mock_user("1", is_staff=True)
        response = RequestMock.do_request_get(
            blob_rest_views.BlobListAdmin.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobListGetPermissions(SimpleTestCase):
    """TestBlobListGetPermissions"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_get(
            blob_rest_views.BlobList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.blob.api.get_all_by_user")
    def test_authenticated_returns_http_200(self, mock_get_all):
        """test_authenticated_returns_http_200

        Args:
            mock_get_all:

        Returns:

        """
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            blob_rest_views.BlobList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch("core_main_app.components.blob.api.get_all_by_user")
    def test_staff_returns_http_200(self, mock_get_all):
        """test_staff_returns_http_200

        Args:
            mock_get_all:

        Returns:

        """
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            blob_rest_views.BlobList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobListPostPermissions(SimpleTestCase):
    """TestBlobListPostPermissions"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_post(
            blob_rest_views.BlobList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(BlobSerializer, "data")
    @patch.object(BlobSerializer, "save")
    @patch.object(BlobSerializer, "is_valid")
    def test_authenticated_returns_http_201(
        self,
        mock_blob_serializer_is_valid,
        mock_blob_serializer_save,
        mock_blob_serializer_data,
    ):
        """test_authenticated_returns_http_201

        Args:
            mock_blob_serializer_is_valid:
            mock_blob_serializer_save:
            mock_blob_serializer_data:

        Returns:

        """
        mock_blob_serializer_is_valid.return_value = True
        mock_blob_serializer_save.return_value = None
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            blob_rest_views.BlobList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(BlobSerializer, "data")
    @patch.object(BlobSerializer, "save")
    @patch.object(BlobSerializer, "is_valid")
    def test_staff_returns_http_201(
        self,
        mock_blob_serializer_is_valid,
        mock_blob_serializer_save,
        mock_blob_serializer_data,
    ):
        """test_staff_returns_http_201

        Args:
            mock_blob_serializer_is_valid:
            mock_blob_serializer_save:
            mock_blob_serializer_data:

        Returns:

        """
        mock_blob_serializer_is_valid.return_value = True
        mock_blob_serializer_save.return_value = None
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            blob_rest_views.BlobList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestBlobDetailGetPermissions(SimpleTestCase):
    """TestBlobDetailGetPermissions"""

    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "get_by_id")
    def test_anonymous_returns_http_200(
        self, mock_blob_api_get_by_id, mock_blob_serializer_data
    ):
        """test_anonymous_returns_http_200

        Args:
            mock_blob_api_get_by_id:
            mock_blob_serializer_data:

        Returns:

        """
        mock_blob_api_get_by_id.return_value = None
        mock_blob_serializer_data.return_value = []

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDetail.as_view(), None, param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "get_by_id")
    def test_authenticated_returns_http_200(
        self, mock_blob_api_get_by_id, mock_blob_serializer_data
    ):
        """test_authenticated_returns_http_200

        Args:
            mock_blob_api_get_by_id:
            mock_blob_serializer_data:

        Returns:

        """
        mock_blob_api_get_by_id.return_value = []
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDetail.as_view(), mock_user, param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "get_by_id")
    def test_staff_returns_http_200(
        self, mock_blob_api_get_by_id, mock_blob_serializer_data
    ):
        """test_staff_returns_http_200

        Args:
            mock_blob_api_get_by_id:
            mock_blob_serializer_data:

        Returns:

        """
        mock_blob_api_get_by_id.return_value = []
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDetail.as_view(), mock_user, param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobAddMetadataPermissions(SimpleTestCase):
    """TestBlobDetailGetPermissions"""

    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "add_metadata")
    @patch.object(blob_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_anonymous_returns_http_403(
        self,
        mock_data_api_get_by_id,
        mock_blob_api_get_by_id,
        mock_blob_api_add_metadata,
        mock_blob_serializer_data,
    ):
        """test_anonymous_returns_http_200

        Args:
            mock_data_api_get_by_id:
            mock_blob_api_get_by_id:
            mock_blob_api_add_metadata:
            mock_blob_serializer_data:

        Returns:

        """
        mock_data_api_get_by_id.return_value = []
        mock_blob_api_get_by_id.return_value = []
        mock_blob_api_add_metadata.return_value = None
        mock_blob_serializer_data.return_value = []

        response = RequestMock.do_request_post(
            blob_rest_views.BlobMetadata.as_view(),
            None,
            param={"pk": "1", "metadata_id": "1"},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "add_metadata")
    @patch.object(blob_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(
        self,
        mock_data_api_get_by_id,
        mock_blob_api_get_by_id,
        mock_blob_api_add_metadata,
        mock_blob_serializer_data,
    ):
        """test_authenticated_returns_http_200

        Args:
            mock_data_api_get_by_id:
            mock_blob_api_get_by_id:
            mock_blob_api_add_metadata:
            mock_blob_serializer_data:

        Returns:

        """
        mock_data_api_get_by_id.return_value = []
        mock_blob_api_get_by_id.return_value = []
        mock_blob_api_add_metadata.return_value = []
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            blob_rest_views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": "1", "metadata_id": "1"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "add_metadata")
    @patch.object(blob_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(
        self,
        mock_data_api_get_by_id,
        mock_blob_api_get_by_id,
        mock_blob_api_add_metadata,
        mock_blob_serializer_data,
    ):
        """test_staff_returns_http_200

        Args:
            mock_data_api_get_by_id:
            mock_blob_api_get_by_id:
            mock_blob_api_add_metadata:
            mock_blob_serializer_data:

        Returns:

        """
        mock_data_api_get_by_id.return_value = []
        mock_blob_api_get_by_id.return_value = []
        mock_blob_api_add_metadata.return_value = []
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            blob_rest_views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": "1", "metadata_id": "1"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobRemoveMetadataPermissions(SimpleTestCase):
    """TestBlobRemoveMetadataPermissions"""

    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "remove_metadata")
    @patch.object(blob_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_anonymous_returns_http_403(
        self,
        mock_data_api_get_by_id,
        mock_blob_api_get_by_id,
        mock_blob_api_remove_metadata,
        mock_blob_serializer_data,
    ):
        """test_anonymous_returns_http_200

        Args:
            mock_data_api_get_by_id:
            mock_blob_api_get_by_id:
            mock_blob_api_remove_metadata:
            mock_blob_serializer_data:

        Returns:

        """
        mock_data_api_get_by_id.return_value = []
        mock_blob_api_get_by_id.return_value = []
        mock_blob_api_remove_metadata.return_value = None
        mock_blob_serializer_data.return_value = []

        response = RequestMock.do_request_delete(
            blob_rest_views.BlobMetadata.as_view(),
            None,
            param={"pk": "1", "metadata_id": "1"},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "remove_metadata")
    @patch.object(blob_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(
        self,
        mock_data_api_get_by_id,
        mock_blob_api_get_by_id,
        mock_blob_api_remove_metadata,
        mock_blob_serializer_data,
    ):
        """test_authenticated_returns_http_200

        Args:
            mock_data_api_get_by_id:
            mock_blob_api_get_by_id:
            mock_blob_api_remove_metadata:
            mock_blob_serializer_data:

        Returns:

        """
        mock_data_api_get_by_id.return_value = []
        mock_blob_api_get_by_id.return_value = []
        mock_blob_api_remove_metadata.return_value = []
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_delete(
            blob_rest_views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": "1", "metadata_id": "1"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "remove_metadata")
    @patch.object(blob_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(
        self,
        mock_data_api_get_by_id,
        mock_blob_api_get_by_id,
        mock_blob_api_remove_metadata,
        mock_blob_serializer_data,
    ):
        """test_staff_returns_http_200

        Args:
            mock_data_api_get_by_id:
            mock_blob_api_get_by_id:
            mock_blob_api_remove_metadata:
            mock_blob_serializer_data:

        Returns:

        """
        mock_data_api_get_by_id.return_value = []
        mock_blob_api_get_by_id.return_value = []
        mock_blob_api_remove_metadata.return_value = []
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_delete(
            blob_rest_views.BlobMetadata.as_view(),
            mock_user,
            param={"pk": "1", "metadata_id": "1"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobDetailDeletePermissions(SimpleTestCase):
    """TestBlobDetailDeletePermissions"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_delete(
            blob_rest_views.BlobDetail.as_view(), None, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(blob_api, "delete")
    @patch.object(blob_api, "get_by_id")
    def test_authenticated_returns_http_204(
        self, mock_blob_api_get_by_id, mock_blob_api_delete
    ):
        """test_authenticated_returns_http_204

        Args:
            mock_blob_api_get_by_id:
            mock_blob_api_delete:

        Returns:

        """
        mock_blob_api_get_by_id.return_value = None
        mock_blob_api_delete.return_value = None

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_delete(
            blob_rest_views.BlobDetail.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(blob_api, "delete")
    @patch.object(blob_api, "get_by_id")
    def test_staff_returns_http_204(
        self, mock_blob_api_get_by_id, mock_blob_api_delete
    ):
        """test_staff_returns_http_204

        Args:
            mock_blob_api_get_by_id:
            mock_blob_api_delete:

        Returns:

        """
        mock_blob_api_get_by_id.return_value = None
        mock_blob_api_delete.return_value = None

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_delete(
            blob_rest_views.BlobDetail.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestBlobDownloadGetPermissions(SimpleTestCase):
    """TestBlobDownloadGetPermissions"""

    @patch.object(blob_api, "get_by_id")
    def test_anonymous_returns_http_200(self, mock_blob_api_get_by_id):
        """test_anonymous_returns_http_200

        Args:
            mock_blob_api_get_by_id:

        Returns:

        """
        mock_blob = Mock()
        mock_blob.blob = BytesIO("blob_text".encode("utf-8"))
        mock_blob.filename = "blob.txt"

        mock_blob_api_get_by_id.return_value = mock_blob

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDownload.as_view(), None, param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(blob_api, "get_by_id")
    def test_authenticated_returns_http_200(self, mock_blob_api_get_by_id):
        """test_authenticated_returns_http_200

        Args:
            mock_blob_api_get_by_id:

        Returns:

        """
        mock_blob = Mock()
        mock_blob.blob = BytesIO("blob_text".encode("utf-8"))
        mock_blob.filename = "blob.txt"

        mock_blob_api_get_by_id.return_value = mock_blob

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDownload.as_view(),
            mock_user,
            param={"pk": "0"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(blob_api, "get_by_id")
    def test_staff_returns_http_200(self, mock_blob_api_get_by_id):
        """test_staff_returns_http_200

        Args:
            mock_blob_api_get_by_id:

        Returns:

        """
        mock_blob = Mock()
        mock_blob.blob = BytesIO("blob_text".encode("utf-8"))
        mock_blob.filename = "blob.txt"

        mock_blob_api_get_by_id.return_value = mock_blob

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDownload.as_view(),
            mock_user,
            param={"pk": "0"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobDeleteListPatchPermissions(SimpleTestCase):
    """TestBlobDeleteListPatchPermissions"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_patch(
            blob_rest_views.BlobDeleteList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(DeleteBlobsSerializer, "validated_data")
    @patch.object(DeleteBlobsSerializer, "is_valid")
    def test_authenticated_returns_http_204(
        self,
        mock_blob_serializer_is_valid,
        mock_blob_serializer_validated_data,
    ):
        """test_authenticated_returns_http_204

        Args:
            mock_blob_serializer_is_valid:
            mock_blob_serializer_validated_data:

        Returns:

        """
        mock_blob_serializer_is_valid.return_value = True
        mock_blob_serializer_validated_data.return_value = list()

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobDeleteList.as_view(), mock_user, data=[]
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(DeleteBlobsSerializer, "validated_data")
    @patch.object(DeleteBlobsSerializer, "is_valid")
    def test_staff_returns_http_204(
        self,
        mock_blob_serializer_is_valid,
        mock_blob_serializer_validated_data,
    ):
        """test_staff_returns_http_204

        Args:
            mock_blob_serializer_is_valid:
            mock_blob_serializer_validated_data:

        Returns:

        """
        mock_blob_serializer_is_valid.return_value = True
        mock_blob_serializer_validated_data.return_value = list()

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobDeleteList.as_view(), mock_user, data=[]
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestBlobAssignPatchPermissions(SimpleTestCase):
    """TestBlobAssignPatchPermissions"""

    @patch.object(blob_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(blob_api, "get_by_id")
    def test_anonymous_returns_http_403(
        self,
        mock_data_api_get_by_id,
        mock_workspace_api_get_by_id,
        mock_data_api_assign,
    ):
        """test_anonymous_returns_http_403

        Args:
            mock_data_api_get_by_id:
            mock_workspace_api_get_by_id:
            mock_data_api_assign:

        Returns:

        """
        mock_data_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_data_api_assign.return_value = None

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobAssign.as_view(),
            None,
            param={"pk": 0, "workspace_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(blob_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(blob_api, "get_by_id")
    def test_authenticated_returns_http_200(
        self,
        mock_blob_api_get_by_id,
        mock_workspace_api_get_by_id,
        mock_blob_api_assign,
    ):
        """test_authenticated_returns_http_200

        Args:
            mock_blob_api_get_by_id:
            mock_workspace_api_get_by_id:
            mock_blob_api_assign:

        Returns:

        """
        mock_blob_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_blob_api_assign.return_value = None

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobAssign.as_view(),
            mock_user,
            param={"pk": 0, "workspace_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(blob_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(blob_api, "get_by_id")
    def test_staff_returns_http_200(
        self,
        mock_blob_api_get_by_id,
        mock_workspace_api_get_by_id,
        mock_blob_api_assign,
    ):
        """test_staff_returns_http_200

        Args:
            mock_blob_api_get_by_id:
            mock_workspace_api_get_by_id:
            mock_blob_api_assign:

        Returns:

        """
        mock_blob_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_blob_api_assign.return_value = None

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobAssign.as_view(),
            mock_user,
            param={"pk": 0, "workspace_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobChangeOwnerPatchPermissions(SimpleTestCase):
    """TestBlobChangeOwnerPatchPermissions"""

    @patch.object(blob_api, "change_owner")
    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Blob, "get_by_id")
    def test_anonymous_returns_http_403(
        self,
        mock_blob_api_get_by_id,
        mock_user_api_get_by_id,
        mock_blob_api_change_owner,
    ):
        """test_anonymous_returns_http_403

        Args:
            mock_blob_api_get_by_id:
            mock_user_api_get_by_id:
            mock_blob_api_change_owner:

        Returns:

        """
        mock_blob_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_blob_api_change_owner.return_value = None

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobChangeOwner.as_view(),
            None,
            param={"pk": 0, "user_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(blob_api, "change_owner")
    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Blob, "get_by_id")
    def test_authenticated_returns_http_403(
        self,
        mock_blob_api_get_by_id,
        mock_user_api_get_by_id,
        mock_blob_api_change_owner,
    ):
        """test_authenticated_returns_http_403

        Args:
            mock_blob_api_get_by_id:
            mock_user_api_get_by_id:
            mock_blob_api_change_owner:

        Returns:

        """
        mock_blob_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_blob_api_change_owner.return_value = None

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobChangeOwner.as_view(),
            mock_user,
            param={"pk": 0, "user_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(blob_api, "change_owner")
    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Blob, "get_by_id")
    def test_staff_returns_http_200(
        self,
        mock_blob_api_get_by_id,
        mock_user_api_get_by_id,
        mock_blob_api_change_owner,
    ):
        """test_staff_returns_http_200

        Args:
            mock_blob_api_get_by_id:
            mock_user_api_get_by_id:
            mock_blob_api_change_owner:

        Returns:

        """
        # Arrange
        # is_staff to access the view
        # is_superuser to be able to change the owner
        user_request = create_mock_user("1", is_staff=True, is_superuser=True)
        mock_blob_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_blob_api_change_owner.return_value = None

        # Act
        response = RequestMock.do_request_patch(
            blob_rest_views.BlobChangeOwner.as_view(),
            user_request,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)



""" SSL utils test class
"""
from unittest import TestCase

from core_main_app.commons.exceptions import SSLError
from core_main_app.utils.requests_utils.ssl import (
    check_ssl_certificates_dir_setting,
)


class TestCheckSslCertificatesDirSetting(TestCase):
    """TestCheckSslCertificatesDirSetting"""

    def test_check_ssl_certificates_dir_setting_ok_when_set_to_true(self):
        """test check ssl certificates dir setting ok when set to True

        Returns:

        """
        check_ssl_certificates_dir_setting(True)

    def test_check_ssl_certificates_dir_setting_ok_when_set_to_false(self):
        """test check ssl certificates dir setting ok when set to False

        Returns:

        """
        check_ssl_certificates_dir_setting(False)

    def test_check_ssl_certificates_dir_setting_raises_error_if_bad_parameter_type(
        self,
    ):
        """test check ssl certificates dir setting raises error if bad parameter type

        Returns:

        """
        with self.assertRaises(SSLError):
            check_ssl_certificates_dir_setting(1)

    def test_check_ssl_certificates_dir_setting_fails_when_set_to_bad_location(
        self,
    ):
        """test check ssl certificates dir setting fails when set to bad location

        Returns:

        """
        fake_dir_path = "./fake/dir"
        with self.assertRaises(SSLError):
            check_ssl_certificates_dir_setting(fake_dir_path)



"""
Template Version Manager model
"""

from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models import F
from core_main_app.commons import exceptions
from core_main_app.components.version_manager.models import VersionManager


class TemplateVersionManager(VersionManager):
    """Manager of templates versions"""

    _class_name = "VersionManager.TemplateVersionManager"
    display_rank = models.IntegerField(null=True, blank=True)

    class Meta:
        ordering = [F("display_rank").asc(nulls_last=True)]

    @property
    def class_name(self):
        return TemplateVersionManager._class_name

    @property
    def version_set(self):
        return self.template_set.all().order_by("pk")

    @staticmethod
    def get_by_id(version_manager_id):
        """Return Version Managers by id.

        Args:
            version_manager_id:

        Returns:

        """
        try:
            return TemplateVersionManager.objects.get(pk=version_manager_id)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_global_version_managers(_cls=True):
        """Return all Template Version Managers with user set to None.

        Returns:
            _cls: if True, restricts to TemplateVersionManager _cls
        """
        queryset = TemplateVersionManager.objects.filter(user=None).all()
        if _cls:
            queryset = queryset.filter(
                _cls=TemplateVersionManager._class_name
            ).all()
        return queryset

    @staticmethod
    def get_active_global_version_manager(_cls=True):
        """Return all active Version Managers with user set to None.

        Returns:

        """
        queryset = TemplateVersionManager.objects.filter(
            is_disabled=False, user=None
        ).all()
        if _cls:
            queryset = queryset.filter(
                _cls=TemplateVersionManager._class_name
            ).all()
        return queryset

    @staticmethod
    def get_active_version_manager_by_user_id(user_id, _cls=True):
        """Return all active Version Managers with given user id.

        Returns:

        """
        if not user_id:
            return TemplateVersionManager.objects.none()
        queryset = TemplateVersionManager.objects.filter(
            is_disabled=False, user=str(user_id)
        ).all()
        if _cls:
            queryset = queryset.filter(
                _cls=TemplateVersionManager._class_name
            ).all()
        return queryset

    @staticmethod
    def get_all_version_manager_by_user_id(user_id, _cls=True):
        """Return all Version Managers with given user id.

        Args:
            user_id:
            _cls:

        Returns:

        """
        if not user_id:
            return TemplateVersionManager.objects.none()
        queryset = TemplateVersionManager.objects.filter(
            user=str(user_id)
        ).all()
        if _cls:
            queryset = queryset.filter(
                _cls=TemplateVersionManager._class_name
            ).all()
        return queryset

    @staticmethod
    def get_all_version_manager(_cls=True):
        """Return all Version Managers.

        Args:
            _cls:

        Returns:

        """
        queryset = TemplateVersionManager.objects.all()
        if _cls:
            queryset = queryset.filter(
                _cls=TemplateVersionManager._class_name
            ).all()
        return queryset

    @staticmethod
    def get_by_id_list(list_id):
        """Return Version Managers with the given id list.

        Args:
            list_id:

        Returns:

        """
        return TemplateVersionManager.objects.filter(pk__in=list_id).all()

    def save_template_version_manager(self):
        """Custom save.

        Returns:
            Saved Instance.

        """
        super().save_version_manager()

    @staticmethod
    def get_active_global_version_manager_by_title(version_manager_title):
        """Return active Template Version Manager by its title with user set to None.

        Args:
            version_manager_title: Version Manager title

        Returns:
            Version Manager instance

        """
        try:
            return TemplateVersionManager.objects.get(
                is_disabled=False, title=version_manager_title, user=None
            )
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))



""" Rights for core main app
"""

# Anonymous group
ANONYMOUS_GROUP = "anonymous"

# Default group
DEFAULT_GROUP = "default"

# API rights
API_CONTENT_TYPE = "api"
API_ACCESS = "api_access"

# Data
# FIXME edit and delete rights are never used
EDIT_DATA = "edit_data"
DELETE_DATA = "delete_data"
PUBLISH_DATA = "publish_data"

# Blob
PUBLISH_BLOB = "publish_blob"

# Form
EDIT_FORM = "edit_form"
DELETE_FORM = "delete_form"

# Workspace
CAN_READ_CODENAME = "can_read_workspace_"
CAN_WRITE_CODENAME = "can_write_workspace_"

CAN_READ_NAME = "Can read workspace "
CAN_WRITE_NAME = "Can write workspace "

CONTENT_TYPE_APP_LABEL = "core_main_app"



""" Unit Test Access Control
"""
from unittest.case import TestCase

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons.exceptions import ApiError
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import create_mock_request
from core_main_app.access_control import api as access_control_api


class TestAccessControlIsSuperuser(TestCase):
    """Test Access Control Is Superuser"""

    def setUp(self):
        """setUp"""
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.superuser = create_mock_user(user_id="1", is_superuser=True)

    def test_access_control_is_superuser_as_anonymous_raises_access_control_error(
        self,
    ):
        """test access control is superuser as anonymous raises access control error

        Returns:

        """
        # Arrange
        mock_request = create_mock_request(user=self.anonymous_user)

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control_api.is_superuser(
                mock_function, request=mock_request
            )

    def test_access_control_is_superuser_as_user_raises_access_control_error(
        self,
    ):
        """test access control is superuser as user raises access control error

        Returns:

        """
        # Arrange
        mock_request = create_mock_request(user=self.user1)

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control_api.is_superuser(
                mock_function, request=mock_request
            )

    def test_access_control_is_superuser_as_superuser_raises_api_error(self):
        """test access control is superuser as super raises api error

        Returns:

        """
        # Arrange
        mock_request = create_mock_request(user=self.superuser)
        exception = ApiError("")

        # Act # Assert
        with self.assertRaises(ApiError):
            access_control_api.is_superuser(
                mock_function, exception, request=mock_request
            )

    def test_access_control_is_superuser_as_superuser_raises_access_control_error(
        self,
    ):
        """test access control is superuser as super raises  access control error

        Returns:

        """
        # Arrange
        mock_request = create_mock_request(user=self.superuser)
        access_control_error = AccessControlError("")

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control_api.is_superuser(
                mock_function, access_control_error, request=mock_request
            )

    def test_access_control_is_superuser_as_superuser_returns_function(self):
        """test access control is superuser as superuser returns function

        Returns:

        """
        # Arrange
        mock_request = create_mock_request(user=self.superuser)

        # Act
        result = access_control_api.is_superuser(
            mock_function, request=mock_request
        )

        # Assert
        self.assertEqual(result, True)


def mock_function(exception=None, *args, **kwargs):
    """setUp

    Args:
        exception:
        args:
        kwargs:

    Returns:
    """
    if exception:
        raise exception
    return True



"""Utils for database backend
"""
from django.conf import settings

POSTGRESQL_BACKEND = "django.db.backends.postgresql_psycopg2"
SQLITE3_BACKEND = "django.db.backends.sqlite3"


def get_default_database_engine():
    """Get default engine

    Returns:

    """
    return settings.DATABASES["default"]["ENGINE"]


def uses_postgresql_backend():
    """Returns True if PostgreSQL backend

    Returns:

    """
    return get_default_database_engine() == POSTGRESQL_BACKEND


def uses_sqlite3_backend():
    """Returns True if SQLite3 backend

    Returns:

    """
    return get_default_database_engine() == SQLITE3_BACKEND



# Generated by Django 3.2.18 on 2023-05-31 20:26

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core_main_app", "0005_template_dependencies"),
    ]

    operations = [
        migrations.AddField(
            model_name="data",
            name="_blob",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="_metadata",
                to="core_main_app.blob",
            ),
        ),
    ]
